/*============================================================================
  File:     Recompilation&PlanGuides.sql

  Summary:  When might a filtered object NOT get used?
  
  SQL Server Version: SQL Server 2008+
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE [AdventureWorksDW2008_ModifiedSalesKey];
GO

EXEC [sp_SQLskills_SQL2012_helpindex] 'dbo.factinternetsales';
EXEC [sp_helpstats] 'dbo.FactInternetSales', 'ALL';
GO

---------------------------------------------
-- What about in a stored procedure?
-- And, you CAN change the code...
---------------------------------------------

IF OBJECT_ID('dbo.GetCustomerSales') IS NOT NULL
	DROP PROCEDURE [dbo].[GetCustomerSales];
GO

CREATE PROCEDURE [dbo].[GetCustomerSales]
(
	@CustomerKey	int
)
AS
SET NOCOUNT ON;

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = @CustomerKey;
GO

EXEC [dbo].[GetCustomerSales] 11509;
GO

-- What about EXECUTE WITH RECOMPILE?
EXEC [dbo].[GetCustomerSales] 11509 WITH RECOMPILE; -- nope
GO

-- What about CREATE WITH RECOMPILE?
ALTER PROCEDURE [dbo].[GetCustomerSales]
(
	@CustomerKey	int
) WITH RECOMPILE
AS
SET NOCOUNT ON;

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = @CustomerKey;
GO

EXEC [dbo].[GetCustomerSales] 11509; -- nope!! 
-- And, I also don't recommend this because this plan does 
-- not go into the plan cache for statement/procedure analysis.
-- And, you really only want to recompile the smallest
-- amount of the code that needs to be recompiled!
GO

-- What OPTION RECOMPILE?
ALTER PROCEDURE [dbo].[GetCustomerSales]
(
	@CustomerKey	int
) 
AS
SET NOCOUNT ON;

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = @CustomerKey
OPTION (RECOMPILE);
GO

EXEC [dbo].[GetCustomerSales] 11509;  -- Success!!


---------------------------------------------
-- What about in a stored procedure?
-- When you CANNOT change the code?
---------------------------------------------

IF OBJECT_ID('dbo.GetCustomerSales') IS NOT NULL
	DROP PROCEDURE [dbo].[GetCustomerSales];
GO

CREATE PROCEDURE [dbo].[GetCustomerSales]
(
	@CustomerKey	int
)
AS
SET NOCOUNT ON;

SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = @CustomerKey;
GO

EXEC [dbo].[GetCustomerSales] 11509;  -- Back to not using FS
GO

EXEC sp_create_plan_guide 
    N'Proc_GetCustomerSales', -- you'll want to come up with a standard naming convention
    N'SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = @CustomerKey', 
    N'OBJECT', 
    N'dbo.GetCustomerSales', 
    NULL, 
    N'OPTION(RECOMPILE)';

EXEC [dbo].[GetCustomerSales] 11509;  -- Success!!
GO


---------------------------------------------
-- What about as an adhoc statement
-- When you CANNOT change the code?
---------------------------------------------

DECLARE @ExecStr	nvarchar(max)
SELECT @ExecStr = 'SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
					FROM [dbo].[FactInternetSales] AS [s] 
						INNER JOIN [dbo].[DimCustomer] AS [c]
							ON [s].[CustomerKey] = [c].[CustomerKey]
					WHERE [c].[CustomerKey] = @CustomerKey'
EXEC sp_executesql @ExecStr
			, N'@CustomerKey int'
			, 11509
GO	-- Back to not using FS

-- Here, you can use a plan guide 

EXEC sp_create_plan_guide 
    N'PreparedStatement_GetCustomerSales', -- come up with a standard naming convention
    'SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
					FROM [dbo].[FactInternetSales] AS [s] 
						INNER JOIN [dbo].[DimCustomer] AS [c]
							ON [s].[CustomerKey] = [c].[CustomerKey]
					WHERE [c].[CustomerKey] = @CustomerKey', 
    N'SQL', 
    NULL, 
    N'@CustomerKey int', 
    N'OPTION(RECOMPILE)';
GO

--SELECT * FROM sys.plan_guides;
--GO

--sp_control_plan_guide N'enable', N'PreparedStatement_GetCustomerSales';
--GO

DECLARE @ExecStr	nvarchar(max)
SELECT @ExecStr = 'SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
					FROM [dbo].[FactInternetSales] AS [s] 
						INNER JOIN [dbo].[DimCustomer] AS [c]
							ON [s].[CustomerKey] = [c].[CustomerKey]
					WHERE [c].[CustomerKey] = @CustomerKey'
EXEC sp_executesql @ExecStr
			, N'@CustomerKey int'
			, 11509
GO	-- SUCCESS!!

---------------------------------------------
-- What if the database is set to forced
-- parameterization?
---------------------------------------------

-- literal/parameter -> sniffed, fs can be used
SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11509;
GO   -- Yep!

ALTER DATABASE [AdventureWorksDW2008_ModifiedSalesKey]
	SET PARAMETERIZATION FORCED;
GO

-- literal/parameter -> sniffed, fs can be used
SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11509;
GO   -- NOPE!!!

DECLARE @FSQuery nvarchar(max);
DECLARE @Parameters nvarchar(max);
EXEC sp_get_query_template 
      N'SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11509',
    @FSQuery OUTPUT, 
    @Parameters OUTPUT;
SELECT @FSQuery, @Parameters
EXEC sp_create_plan_guide 
    N'AdhocTemplate_FS_GetCustomerSales', -- you'll want to come up with a standard naming convention
    @FSQuery, 
    N'TEMPLATE', 
    NULL, 
    @Parameters, 
    N'OPTION(PARAMETERIZATION SIMPLE)';
GO

-- literal/parameter -> sniffed, fs can be used
SELECT [c].[CustomerKey], [c].[LastName], [s].[SalesAmount]
FROM [dbo].[FactInternetSales] AS [s] 
	INNER JOIN [dbo].[DimCustomer] AS [c]
		ON [s].[CustomerKey] = [c].[CustomerKey]
WHERE [c].[CustomerKey] = 11509;
GO   -- SUCCESS!!

---------------------------------------------
-- Final cleanup!
---------------------------------------------

ALTER DATABASE [AdventureWorksDW2008_ModifiedSalesKey]
	SET PARAMETERIZATION SIMPLE;
GO

SELECT * FROM sys.plan_guides;
GO

EXEC sp_control_plan_guide 'drop all';
GO
